<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Class to manage a logging system.
 * It implements just a static function (doLog).
 * See $config['SYSTEM']['ALERT_LEVEL'] in mymvc.php
 * to configure the alert level (when should be logged)
 */
class myMVC_Logger
{
  /**
   * Possible levels
   */
  static $levelString = array(
                              ALERT_SQL       => 'SQL',
                              ALERT_WARNING   => 'WARNING',
                              ALERT_DEBUG     => 'DEBUG',
                              ALERT_CRITICAL  => 'CRITICAL',
                             );

  /**
   * Save a message in the error log
   * 
   * @param string $method      Method of the class
   * @param int $line           Line that generate the error
   * @param string $message     The error message
   * @param int $level          Error level (see mymvc.php)
   * @param boolean $send       true if the error should be sent per E-Mail oder to a Nagios server (Default: false)
   * @return string             The complete error message, to use for an Exception
   */
  public static function doLog($method, $line, $message, $level = ALERT_CRITICAL, $send = false)
  {
    global $config;

    $hasToLog = false;
    $msg = '';
    if($level >= $config['SYSTEM']['ALERT_LEVEL'])
      $hasToLog = true;
    if($level == ALERT_SQL && $config['SYSTEM']['DEBUG_SQL'])
      $hasToLog = true;

    if($hasToLog)
    {
      $msg = sprintf("%s: %s (line %d) - %s - %s\n",
                    date('[y-m-d H:i:s]'),
                    $method, $line,
                    myMVC_Logger::$levelString[$level],
                    $message);
      @error_log($msg, 3, $config['SYSTEM']['LOG_FILE']);
      if($send)
        myMVC_ErrorHandler::sendAlert($msg, $level);
    }

    return $msg;
  }
}
?>
