<?php
/**
 * DB-Class für account
 */
class Account extends myMVC_BaseLogin
{
  public static $filtersTests         = array('body' => '$message_body',
                                              'from (header)' => '$header_from',
                                              'to (header)' => '$header_to',
                                              'cc (header)' => '$header_cc',
                                              'sender' => '$sender_address',
                                              'recipient' => '$local_part',
                                              'size' => '$message_size',
                                              'subject' => '$header_subject:',
                                              'headers' => '$message_headers',
                                              'list-id' => '$header_List-Id:',
                                              'DK status' => '$header_DomainKey-Status:',
                                              'SPF status' => 'Received-SPF:');
  public static $filtersConditions    = array('is', 'is not', 'contains', 'does not contains',
                                              'begins', 'does not begins', 'ends', 'does not ends',
                                              'is above', 'is not above', 'is below', 'is not below');
  public static $filtersActions       = array('deliver' => array('deliver', 'REGEXP:/^[A-z0-9][\w.-]*@[A-z0-9][\w\-\.]+\.[A-z0-9]{2,6}$/', 1),
                                              'discard' => array('seen finish', 'NONE', 0),
                                              'save'    => array('save', 'REGEXP:/^[A-z0-9][\w.-]*$/', 1));
  private static $searchOrder         = "accountType = 'MBOX' DESC, accountType = 'FORWARD' DESC, accountType = 'PIPE' DESC, accountname ASC";

  public function __construct()
  {
    parent::__construct('account',
                        array('accountname'           => 'Account::checkString255NotNull',
                              'passwd'                => 'Account::checkString255Null',
                              'accountType'           => 'Account::checkAccountType',
                              'dataHam'               => 'Account::checkTextNotNull',
                              'dataSpam'              => 'Account::checkTextNull',
                              'filters'               => 'Account::checkTextNull',
                              'hasAutoresponder'      => 'Account::checkBoolNotNull',
                              'autoresponderSubject'  => 'Account::checkString255Null',
                              'autoresponderText'     => 'Account::checkTextNull',
                              'finaldest'             => 'Account::checkString255Null',
                              'finaldestSpam'         => 'Account::checkString255Null',
                              'virtual'               => 'Account::checkBoolNotNull',
                              'uid'                   => 'Account::checkSignedIntNotNull',
                              'spamScanType'          => 'Account::checkSpamScanType',
                              'virusScanType'         => 'Account::checkVirusScanType',
                              'useSPF'                => 'Account::checkBoolNotNull',
                              'useGreylist'           => 'Account::checkBoolNotNull',
                              'useDomainKey'          => 'Account::checkBoolNotNull',
                              'isAdmin'               => 'Account::checkIsAdmin',
                              'toActivate'            => 'Account::checkBoolNotNull',
                              'toDelete'              => 'Account::checkBoolNotNull',
                              'canModifyAlias'        => 'Account::checkBoolNotNull',
                              'canSetAutoresponder'   => 'Account::checkBoolNotNull',
                              'canUseFilters'         => 'Account::checkBoolNotNull',
                              'rejectedSpamScore'     => 'Account::checkSignedSmallIntNotNull',
                              'rejectAfterSize'       => 'Account::checkSignedSmallIntNull',
                        ),
                        array('accountname' => 'pk'),
                        array());
    $this->registerFinder('findAll', array(), array());
    $this->registerFinder('findByAccountType', array('accountType' => 'EQUAL'), array());
    $this->registerCounter('getCountAll', array(), array());
    $this->registerCounter('getCountByAccountType', array('accountType' => 'EQUAL'), array());
  }

  public static function getAccountTypes()
  {
  	return array('MBOX' => _('Mailbox'), 'FORWARD' => _('Weiterleitung'), 'PIPE' => _('Mailroboter'));
  }

  public static function getSpamScanTypes()
  {
    return array('INACTIVE' => _('Deaktiviert'), 'SPAMFOLDER' => _('Verschieben in Spam-Ordner'), 'MARK' => _('Markierung in Header'), 'SUBJECT' => _('Markierung im Betreff'));
  }

  public static function getVirusScanTypes()
  {
    return array('INACTIVE' => _('Deaktiviert'), 'SPAMFOLDER' => _('Verschieben in Spam-Ordner'), 'REJECT' => _('Ablehnung mit 5xx-Fehler'), 'SUBJECT' => _('Markierung im Betreff'));
  }
  
  public static function getAdminTypes($userType = 't')
  {
    switch($userType)
    {
      case 'f':
        return array('f' => _('Normaler Nutzer'));
      case 'd':
        return array('d' => _('Domainadministrator'), 'f' => _('Normaler Nutzer'));
      case 't':
        return array('t' => _('System-Administrator'), 'd' => _('Domainadministrator'), 'f' => _('Normaler Nutzer'));
    }
    return array('t' => _('System-Administrator'), 'd' => _('Domainadministrator'), 'f' => _('Normaler Nutzer'));
  }

  public function getTypeInformation()
  {
    switch($this->getIsAdmin())
    {
      case 't':
        return _('Als Serveradministrator k&ouml;nnen Sie alle Domain und Konten auf dem Server verwalten. Sie k&ouml;nnen Konten anlegen, l&ouml;schen und &auml;ndern, Black- und Whitelisten aller Konten verwalten, Domain anlegen und l&ouml;schen, und Domain- und Serveradministratoren einrichten.');
      case 'd':
        return _('Als Domainadministrator k&ouml;nnen Sie nur Ihre Domain verwalten. Sie k&ouml;nnen Konten anlegen, l&ouml;schen und &auml;ndern (aber nur innerhalb Ihrer Domain), Black- und Whitelisten aller Konten Ihrer Domain verwalten,  und Domainadministratoren f&uuml;r Ihre Domain einrichten.');
      case 'f':
        return _('Als normaler Nutzer d&uuml;rfen Sie nur Ihr Konto verwalten.');
    }
    return '';
  }

  /**
   * Gibt die Filters des Nutzers zurück
   * 
   * @return array                              Die Filter
   */
  public function getParsedFilters()
  {
    $filters = array();
    foreach(explode('if', $this->getFilters()) as $f)
    {
      if(chop($f) != '')
      {
        $f = ereg_replace("end$", " endif", $f);
        $f = str_replace('%FINALDEST%/.', '', $f);
        array_push($filters, "if $f");
      }
    }
    return $filters;
  }

  /**
   * Gibt die menschlich lesbare Version des Filters zurück
   * 
   * @param string $filter                      Der Filter
   * @return string                             Die lesbare Version
   */
  public static function parseFilter($filter)
  {
    foreach(array_keys(self::$filtersTests) as $k)
      $filter = str_replace(self::$filtersTests[$k], $k, $filter);

    return $filter;
  }

  /**
   * Berechnet den Pfad zur Mailbox für den Nutzer
   * 
   * @param string $type                  Mailboxtyp (mail oder spam)
   * @param string $domain                Optionale Domain des Nutzers (falls die Mailboxes die Domain enthalten sollen)
   * @return string                       Berechneter Pfad des Mailboxes
   */
  public function getMailbox($type = 'mail', $domain = '')
  {
    global $config;

    $path = $config['MBOXCONFIG']['PATH'];
    $path = str_replace('%%TYPE%%', 'mail', $path);
    $path = str_replace('%%ACCOUNT%%', $this->getAccountname(), $path);
    $path = str_replace('%%DOMAIN%%', $domain, $path);
    $path .= '/'.$config['MBOXCONFIG']['BASE'];
    if($type == 'spam')
      $path .= '/'.$config['MBOXCONFIG']['SPAM'];
  
    return $path;
  }

  /**
   * Gibt die Nutzer, die der Nutzer administriert
   *
   * @param string $search                Das Suchkriterium. NULL um alle Nutzer zu holen
   * @param boolean $alsoInactive         Wenn true, werden auch Nutzer zurückgegeben, die noch zu aktivieren sind
   * @return array                        Die administrierte Nutzer
   */
  public function getUsersForAdmin($search = NULL, $alsoInactive = false)
  {
    $ret = array();
// sollte nicht passieren, aber wenn der Nutzer kein Admin ist, leere Liste zurückgeben
    if($this->getIsAdmin() == 'f')
      return array();
    if($this->getIsAdmin() == 't') // System-Administratoren sehen alles
    {
      if($alsoInactive)
      {
        if($search === NULL)
          $sql = sprintf("SELECT DISTINCT * FROM account WHERE toDelete = 'f' ORDER BY %s", self::$searchOrder);
        else
          $sql = sprintf("SELECT DISTINCT * FROM account WHERE toDelete = 'f' AND accountname LIKE %s ORDER BY %s",
              $this->engine->quoteSqlStringForLike($search), self::$searchOrder);
      }
      else
      {
        if($search === NULL)
          $sql = sprintf("SELECT DISTINCT * FROM account WHERE toDelete = 'f' AND toActivate = 'f' ORDER BY %s", self::$searchOrder);
        else
          $sql = sprintf("SELECT DISTINCT * FROM account WHERE toDelete = 'f' AND toActivate = 'f' AND accountname LIKE %s ORDER BY %s",
              $this->engine->quoteSqlStringForLike($search), self::$searchOrder);
      }
    }
    else
    {
      if($alsoInactive)
      {
        if($search === NULL)
          $sql = sprintf("SELECT DISTINCT account.*
                          FROM account
                          JOIN aliases USING (accountname)
                          JOIN (
                            SELECT DISTINCT aliases.domainname
                            FROM aliases WHERE aliases.accountname = %s
                          ) a ON (aliases.domainname = a.domainname)
                          WHERE account.isAdmin IN ('f', 'd') AND toDelete = 'f'
                          ORDER BY %s",
              $this->engine->quoteSqlString($this->getAccountname()),
              self::$searchOrder);
        else
          $sql = sprintf("SELECT DISTINCT account.*
                          FROM account
                          JOIN aliases USING (accountname)
                          JOIN (
                            SELECT DISTINCT aliases.domainname
                            FROM aliases WHERE aliases.accountname = %s
                          ) a ON (aliases.domainname = a.domainname)
                          WHERE account.isAdmin IN ('f', 'd') AND account.accountname LIKE %s AND toDelete = 'f' 
                          ORDER BY %s",
              $this->engine->quoteSqlString($this->getAccountname()),
              $this->engine->quoteSqlStringForLike($search),
              self::$searchOrder);
      }
      else
      {
        if($search === NULL)
          $sql = sprintf("SELECT DISTINCT account.*
                          FROM account
                          JOIN aliases USING (accountname)
                          JOIN (
                            SELECT DISTINCT aliases.domainname
                            FROM aliases WHERE aliases.accountname = %s
                          ) a ON (aliases.domainname = a.domainname)
                          WHERE account.isAdmin IN ('f', 'd') AND toDelete = 'f' AND toActivate = 'f'
                          ORDER BY %s",
              $this->engine->quoteSqlString($this->getAccountname()),
              self::$searchOrder);
        else
          $sql = sprintf("SELECT DISTINCT account.*
                          FROM account
                          JOIN aliases USING (accountname)
                          JOIN (
                            SELECT DISTINCT aliases.domainname
                            FROM aliases WHERE aliases.accountname = %s
                          ) a ON (aliases.domainname = a.domainname)
                          WHERE account.isAdmin IN ('f', 'd') AND account.accountname LIKE %s AND toDelete = 'f' AND toActivate = 'f' 
                          ORDER BY %s",
              $this->engine->quoteSqlString($this->getAccountname()),
              $this->engine->quoteSqlStringForLike($search),
              self::$searchOrder);
      }
    }
    $res = $this->findBySQLQuery($sql);
    while($row = $this->engine->getValueArrayFromResult($res))
    {
      $rowClass = new Account();
      $rowClass->saveValues($row);
      array_push($ret, $rowClass);
    }
    $this->engine->freeResult($res);
    return $ret;
  }

  /**
   * Gibt die Parameter für die ausgewählte PIPE zurück
   *  
   * @param string $pipeName              Die PIPE
   * @return array                        Die Parameter als Array von String
   * @throws myMVC_DatabaseException      Bei Probleme mit der Datenbank
   */
  public static function getParametersForPipe($pipeName)
  {
    global $config;

    $params = array();
  
    foreach($config['MBOXCONFIG']['PIPES'] as $pipe)
    {
      if($pipe['name'] == $pipeName)
      {
        list($type, $v) = explode(':', $pipe['values']);
        list($prefix, $other) = explode('|', $v, 2);
        if($other == '' && $prefix != '')
        {
          $other = $prefix;
          $prefix = '';
        }
        switch($type)
        {
          case 'PIPE':
            $fp = popen($other, 'r');
            while(!feof($fp))
            {
              if($buf = trim(fgets($fp, 4096)))
                array_push($params, $buf);
            }
            pclose($fp);
          break;
          case 'FILE':
            $fp = fopen($other, 'r');
            while(!feof($fp))
            {
              if($buf = trim(fgets($fp, 4096)))
                array_push($params, $buf);
            }
            fclose($fp);
          break;
          case 'DB':
            list($dbHost, $dbName, $dbUser, $dbPass, $query) = explode('/', $other);
            $myLink = @mysql_connect($dbHost, $dbUser, $dbPass);
            if($myLink === false)
              throw new myMVC_DatabaseException('Could not connect to DB');
            if(@mysql_select_db($dbName, $myLink) === false)
              throw new myMVC_DatabaseException('Could not select database');
            $result = @mysql_query($query, $myLink);
            while($rs = @mysql_fetch_array($result))
              array_push($params, $rs[0]);
            @mysql_free_result($result);
            @mysql_close($myLink);
          break;
          case 'FIX':
            $params = explode('/', $other);
          break;
        }
      }
    }

    return($params);
  }

  /**
   * Check the given password against the saved and returns, if it is correct.
   * 
   * @param string $passwd                      Account's given password
   * @return boolean                            true if the password was correct, false otherwise
   */
  public function isPasswordCorrect($passwd)
  {
    if($passwd == $this->getPassword())
      return true;
    return false;
  }

  /**
   * Check function for TEXT NOT NULL
   */
  public static function checkTextNotNull($value, $type)
  {
    if($value === NULL)
      return false;

    return !(!is_string($value) || strlen($value) == 0 || strlen($value) > 65535);
  }

  /**
   * Check function for TEXT NULL
   */
  public static function checkTextNull($value, $type)
  {
    if($value === NULL)
      return true;

    return !(!is_string($value) || strlen($value) > 65535);
  }

  public static function checkAccountType($value, $type)
  {
    if(in_array($value, array_keys(self::getAccountTypes())))
      return true;
    return false;
  }

  public static function checkSpamScanType($value, $type)
  {
    if(in_array($value, array_keys(self::getSpamScanTypes())))
      return true;
    return false;
  }

  public static function checkVirusScanType($value, $type)
  {
    if(in_array($value, array_keys(self::getVirusScanTypes())))
      return true;
    return false;
  }

  public static function checkIsAdmin($value, $type)
  {
    if(in_array($value, array_keys(self::getAdminTypes())))
      return true;
    return false;
  }
}
?>